'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _fs = require('./fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _datNode = require('dat-node');

var _datNode2 = _interopRequireDefault(_datNode);

var _logger = require('./logger');

var _logger2 = _interopRequireDefault(_logger);

var _pipeStreams = require('./pipe-streams');

var _pipeStreams2 = _interopRequireDefault(_pipeStreams);

var _formatSize = require('./format-size');

var _storage = require('./storage');

var _storage2 = _interopRequireDefault(_storage);

var _prompt = require('./prompt');

var _prompt2 = _interopRequireDefault(_prompt);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

class DatCp {

  constructor(program, options = {}) {
    this.program = program;
    this.options = options;
    this.files = 0;
    this.totalSize = 0;
  }

  connect() {
    var _this = this;

    return new Promise(resolve => {
      _logger2.default.debug('Creating dat archive.');

      (0, _datNode2.default)((0, _storage2.default)('.'), _extends({}, this.options), (() => {
        var _ref = _asyncToGenerator(function* (err, dat) {
          if (err) {
            _logger2.default.error(err.toString());
            _logger2.default.error(`Failed to initialize dat archive.`);
            process.exit(1);
          }

          _this.dat = dat;
          dat.trackStats();

          yield _this.joinNetwork();
          resolve();
        });

        return function (_x, _x2) {
          return _ref.apply(this, arguments);
        };
      })());
    });
  }

  joinNetwork() {
    return new Promise((resolve, reject) => {
      _logger2.default.debug('Connecting to dat network.');

      this.dat.joinNetwork();

      if (!this.options.key) {
        return resolve();
      }

      const abort = setTimeout(() => {
        _logger2.default.error('Failed to connect to any peers.');
        process.exit(1);
      }, 15000);

      const connect = setInterval(() => {
        for (const conn of this.dat.network.connections) {
          if (conn.writable) {
            _logger2.default.debug('Connected to upload peer.');
            clearInterval(connect);
            clearTimeout(abort);
            resolve();
          }
        }
      }, 300);
    });
  }

  upload(paths) {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      yield _this2.ensurePathsValid(paths);

      if (_this2.program.dryRun) {
        _this2.printTotal();
        process.exit(0);
      }

      _logger2.default.debug('Creating metadata for files:');
      for (const path of paths) {
        yield _this2.uploadPath(path, '');
      }

      _this2.printTotal();
    })();
  }

  ensurePathsValid(paths) {
    var _this3 = this;

    return _asyncToGenerator(function* () {
      _this3.count = 0;

      for (const path of paths) {
        yield _this3.ensurePathValid(path);
      }

      if (!_this3.files) {
        _logger2.default.error(`No files to copy.`);
        process.exit(1);
      }
    })();
  }

  ensurePathValid(path) {
    var _this4 = this;

    return _asyncToGenerator(function* () {
      let stats;

      try {
        stats = yield _fs2.default.lstat(path);
      } catch (err) {
        _logger2.default.error(`${path}: No such file or directory.`);
        process.exit(1);
      }

      if (stats.isFile()) {
        _this4.countPath(path, stats, _this4.program.dryRun);
        return;
      }

      if (stats.isDirectory()) {
        return yield _this4.ensureDirValid(path, stats);
      }

      _logger2.default.warn(`${path}: Not a file or directory (not copied).`);
      return;
    })();
  }

  ensureDirValid(path, stats) {
    var _this5 = this;

    return _asyncToGenerator(function* () {
      if (!_this5.program.recursive) {
        _logger2.default.warn(`${path}: Is a directory (not copied).`);
        return;
      }

      if (path[path.length - 1] !== '/') {
        _this5.countPath(path, stats, _this5.program.dryRun);
      }

      const dirPaths = yield _fs2.default.readdir(path);

      for (const dirPath of dirPaths) {
        yield _this5.ensurePathValid(_path2.default.join(path, dirPath));
      }
    })();
  }

  uploadPath(path, datPath) {
    var _this6 = this;

    return _asyncToGenerator(function* () {
      const stats = yield _fs2.default.lstat(path);

      if (stats.isFile()) {
        yield _this6.uploadFile(path, datPath);
      } else if (stats.isDirectory()) {
        yield _this6.uploadDir(path, datPath);
      }
    })();
  }

  uploadFile(path, datPath) {
    var _this7 = this;

    return _asyncToGenerator(function* () {
      datPath = _path2.default.join(datPath, _path2.default.parse(path).base);
      const stats = yield _fs2.default.lstat(path);
      const filesize = stats.size || 1;

      const readStream = _fs2.default.createReadStream(path);
      const writeStream = _this7.dat.archive.createWriteStream(datPath, { path });

      yield (0, _pipeStreams2.default)(readStream, writeStream, filesize, datPath);
    })();
  }

  uploadDir(path, datPath) {
    var _this8 = this;

    return _asyncToGenerator(function* () {
      if (!_this8.program.recursive) {
        return;
      }

      // If a source dir ends with `/`, copy its contents, not the dir itself
      if (path[path.length - 1] !== '/') {
        datPath = _path2.default.join(datPath, _path2.default.parse(path).base);
        yield _this8.mkdir(datPath);
      }

      const dirPaths = yield _fs2.default.readdir(path);

      for (const dirPath of dirPaths) {
        yield _this8.uploadPath(_path2.default.join(path, dirPath), datPath);
      }
    })();
  }

  mkdir(path) {
    return new Promise(resolve => {
      this.dat.archive.mkdir(path, err => {
        if (err) {
          _logger2.default.error(err.toString());
          _logger2.default.error(`${path}: Failed to create directory in dat archive.`);
          process.exit(1);
        }
        resolve();
      });
    });
  }

  download(listOnly = false) {
    var _this9 = this;

    return new Promise(resolve => {
      const abort = setTimeout(() => {
        _logger2.default.error('Failed to download metadata from peer.');
        process.exit(1);
      }, 15000);

      const readRoot = (() => {
        var _ref2 = _asyncToGenerator(function* () {
          const paths = yield _this9.readdir('/');

          if (paths.length !== 0) {
            clearTimeout(abort);

            for (const path of paths) {
              yield _this9.downloadPath(path, listOnly);
            }

            if (_this9.program.dryRun || !listOnly && _this9.files > 30 || _this9.files === 0) {
              _this9.printTotal();
            }
            resolve();
          } else {
            setTimeout(readRoot, 300);
          }
        });

        return function readRoot() {
          return _ref2.apply(this, arguments);
        };
      })();

      readRoot();
    });
  }

  downloadPath(path, listOnly) {
    var _this10 = this;

    return _asyncToGenerator(function* () {
      const stats = yield _this10.stat(path);

      if (stats.isDirectory()) {
        yield _this10.downloadDir(path, stats, listOnly);
      } else {
        yield _this10.downloadFile(path, stats, listOnly);
      }
    })();
  }

  downloadFile(path, stats, listOnly) {
    var _this11 = this;

    return _asyncToGenerator(function* () {
      // If the file exists and is the same size, assume that it hasn't changed
      // and skip it.
      try {
        const fsStats = yield _fs2.default.lstat(path);
        if (stats.size === fsStats.size) {
          _logger2.default.warn(`${path}: File is identical (not copied).`);
          return;
        }
      } catch (err) {
        // File doesn't exist, do nothing.
      }

      _this11.countPath(path, stats, listOnly);

      if (listOnly) {
        return;
      }

      const readStream = _this11.dat.archive.createReadStream(path);
      const writeStream = _fs2.default.createWriteStream(path);
      const filesize = stats.size || 1;

      yield (0, _pipeStreams2.default)(readStream, writeStream, filesize, path);
    })();
  }

  downloadDir(path, stats, listOnly) {
    var _this12 = this;

    return _asyncToGenerator(function* () {
      if (!listOnly) {
        // lstat will throw an error if a path does not exist, so rely on that to
        // know that the dir does not already exist. If the path exists and is not
        // a directory, error.
        try {
          const stats = yield _fs2.default.lstat(path);
          if (!stats.isDirectory()) {
            _logger2.default.error(`${path}: Not a directory.`);
            process.exit(1);
          }
        } catch (err) {
          yield _fs2.default.mkdir(path);
        }
      }

      _this12.countPath(path, stats, listOnly);

      const dirPaths = yield _this12.readdir(path);

      for (const dirPath of dirPaths) {
        yield _this12.downloadPath(_path2.default.join(path, dirPath), listOnly);
      }
    })();
  }

  countPath(path, stats, listPath) {
    if (path === '.') {
      return;
    }

    this.files += 1;
    this.totalSize += stats.size;

    if (!listPath) {
      return;
    }

    if (stats.isFile()) {
      _logger2.default.info(`${(0, _formatSize.formatSize)(stats.size).padEnd(8)} ${path}`);
    } else if (stats.isDirectory()) {
      _logger2.default.info(`${'-'.padEnd(8)} ${path}`);
    }
  }

  printTotal() {
    _logger2.default.info(`\nTotal: ${this.files} files (${(0, _formatSize.formatSize)(this.totalSize)})`);
  }

  resetCounts() {
    this.files = 0;
    this.totalSize = 0;
  }

  downloadPrompt() {
    var _this13 = this;

    return _asyncToGenerator(function* () {
      const answer = yield (0, _prompt2.default)(`\nDownload ${_this13.files} files (${(0, _formatSize.formatSize)(_this13.totalSize)})? [Y/n] `);

      const proceed = ['yes', 'y', ''].includes(answer.trim().toLowerCase());

      if (proceed) {
        _logger2.default.info();
      }

      return proceed;
    })();
  }

  readdir(path) {
    return new Promise((resolve, reject) => {
      this.dat.archive.readdir(path, (() => {
        var _ref3 = _asyncToGenerator(function* (err, paths) {
          if (err) {
            _logger2.default.error(err.toString());
            _logger2.default.error(`${path}: Failed to read from dat archive.`);
            process.exit(1);
          }
          resolve(paths);
        });

        return function (_x3, _x4) {
          return _ref3.apply(this, arguments);
        };
      })());
    });
  }

  stat(path) {
    return new Promise((resolve, reject) => {
      this.dat.archive.stat(path, (err, stats) => {
        if (err) {
          _logger2.default.error(err.toString());
          _logger2.default.error(`${path}: Failed to get stats from dat archive.`);
          process.exit(1);
        }
        resolve(stats);
      });
    });
  }

}
exports.default = DatCp;